#pps-loading-message {
    background-color: @theme_bg_color;
    color: @theme_fg_color;
    border-radius: 10px;
    padding: 10px;
}

.content-view.document-page {
    box-shadow: alpha(@card_shade_color, 0.75) 0px 2px 3px 1px, @card_shade_color 0px 1px 3px 1px;
}

window.pps-annotation-window {
    padding-left: 2px;
    padding-right: 2px;
    padding-bottom: 2px;
    border-bottom-left-radius: 0px;
    border-bottom-right-radius: 0px;
}

pps-view:selected {
    background-color: @theme_selected_bg_color;
}

pps-view.find-results:selected {
    background-color: alpha(@accent_bg_color, 0.3);
}

pps-view.find-results:active {
    background-color: alpha(@accent_bg_color, 0.6);
}

pps-view.document-page {
    background-color: white;
    padding: 0;
}

pps-view.document-page.inverted {
    background-color: black;
}

.inverted-color {
    filter: invert(1);
}

pps-presentation-view {
    background-color: black;
}

pps-presentation-view.white-mode {
    background-color: white;
}

.toolbar.nested {
  padding-top: 0px;
  padding-bottom: 0px;
}

.content-view .combo {
    padding-left: 2px;
    padding-right: 2px;
}
