.content-view.document-page {
    box-shadow: alpha(@card_shade_color, 0.75) 0px 2px 3px 1px, @card_shade_color 0px 1px 3px 1px;
}

PpsView {
    background-color: @theme_bg_color;
}

PpsView:selected {
    background-color: @theme_selected_bg_color;
}

PpsView.document-page {
    background-color: white;
    padding: 0;
}
